Intersection

Struct Intersection 

pub struct Intersection<L, R> {
    pub point: [f64; 2],
    pub left: L,
    pub right: R,
}
Intersection point and indices of the “left” and “right” composite types.

This struct is used as the Iterator::Item of the iterators created by the intersection methods of the Composite trait. It contains the intersection point itself as well as the segment keys of the involved composite types. These keys (SegmentIdx or ShapeIdx) can be used to retrieve the segments from the composite types where the intersection occurred (see Composite::segment).

Fields

point: [f64; 2]

The intersection point itself.

left: L

Key to retrieve the intersected segment of the “left” side composite (self for the intersection functions in Composite).

right: R

Key to retrieve the intersected segment of the “right” side composite (the second argument of the intersection functions in Composite).

Implementations

Source

impl<L, R> Intersection<L, R>

pub fn switch(self) -> Intersection<R, L>

Switches the left and ``rightfields ofSelf`.

Source

impl Intersection<ShapeIdx, ShapeIdx>

pub fn draw( &self, style: &IntersectionStyle, left: Option<(&Shape, &Style)>, right: Option<(&Shape, &Style)>, context: &Context, ) -> Result<(), Error>

Draws the intersection between two Shapes onto the given cairo::Context using the given IntersectionStyle.

Optionally, the intersected segments of the involved composite types SegmentChain, Contour or Shape can also be drawn onto the canvas if the corresponding (&Composite, &Style) tuple is provided. See the module documentation for an example.

Source

impl Intersection<ShapeIdx, SegmentIdx>

pub fn draw( &self, style: &IntersectionStyle, left: Option<(&Shape, &Style)>, right: Option<(&SegmentChain, &Style)>, context: &Context, ) -> Result<(), Error>

Draws the intersection between a Shape and a SegmentChain onto the given cairo::Context using the given IntersectionStyle.

Optionally, the intersected segments of the involved composite types SegmentChain, Contour or Shape can also be drawn onto the canvas if the corresponding (&Composite, &Style) tuple is provided. See the module documentation for an example.

Source

impl Intersection<SegmentIdx, ShapeIdx>

pub fn draw( &self, style: &IntersectionStyle, left: Option<(&SegmentChain, &Style)>, right: Option<(&Shape, &Style)>, context: &Context, ) -> Result<(), Error>

Draws the intersection between a SegmentChain and a Shape onto the given cairo::Context using the given IntersectionStyle.

Optionally, the intersected segments of the involved composite types SegmentChain, Contour or Shape can also be drawn onto the canvas if the corresponding (&Composite, &Style) tuple is provided. See the module documentation for an example.

Source

impl Intersection<ShapeIdx, ()>

pub fn draw( &self, style: &IntersectionStyle, left: Option<(&Shape, &Style)>, context: &Context, ) -> Result<(), Error>

Draws the intersection between a Shape and a Primitive onto the given cairo::Context using the given IntersectionStyle.

Optionally, the intersected segments of the involved composite types SegmentChain, Contour or Shape can also be drawn onto the canvas if the corresponding (&Composite, &Style) tuple is provided. See the module documentation for an example.

Source

impl Intersection<(), ShapeIdx>

pub fn draw( &self, style: &IntersectionStyle, right: Option<(&Shape, &Style)>, context: &Context, ) -> Result<(), Error>

Draws the intersection between a Primitive and a Shape onto the given cairo::Context using the given IntersectionStyle.

Optionally, the intersected segments of the involved composite types SegmentChain, Contour or Shape can also be drawn onto the canvas if the corresponding (&Composite, &Style) tuple is provided. See the module documentation for an example.

Source

impl Intersection<SegmentIdx, SegmentIdx>

pub fn draw( &self, style: &IntersectionStyle, left: Option<(&SegmentChain, &Style)>, right: Option<(&SegmentChain, &Style)>, context: &Context, ) -> Result<(), Error>

Draws the intersection between two SegmentChains onto the given cairo::Context using the given IntersectionStyle.

Optionally, the intersected segments of the involved composite types SegmentChain, Contour or Shape can also be drawn onto the canvas if the corresponding (&Composite, &Style) tuple is provided. See the module documentation for an example.

Source

impl Intersection<SegmentIdx, ()>

pub fn draw( &self, style: &IntersectionStyle, left: Option<(&SegmentChain, &Style)>, context: &Context, ) -> Result<(), Error>

Draws the intersection between a SegmentChain and a Primitive onto the given cairo::Context using the given IntersectionStyle.

Optionally, the intersected segments of the involved composite types SegmentChain, Contour or Shape can also be drawn onto the canvas if the corresponding (&Composite, &Style) tuple is provided. See the module documentation for an example.

Source

impl Intersection<(), SegmentIdx>

pub fn draw( &self, style: &IntersectionStyle, right: Option<(&SegmentChain, &Style)>, context: &Context, ) -> Result<(), Error>

Draws the intersection between a Primitive and a SegmentChain onto the given cairo::Context using the given IntersectionStyle.

Optionally, the intersected segments of the involved composite types SegmentChain, Contour or Shape can also be drawn onto the canvas if the corresponding (&Composite, &Style) tuple is provided. See the module documentation for an example.

Source

impl Intersection<(), ()>

pub fn draw( &self, style: &IntersectionStyle, context: &Context, ) -> Result<(), Error>

Draws the intersection between two Primitives onto the given cairo::Context using the given IntersectionStyle.

Optionally, the intersected segments of the involved composite types SegmentChain, Contour or Shape can also be drawn onto the canvas if the corresponding (&Composite, &Style) tuple is provided. See the module documentation for an example.