Geometry

Enum Geometry 

pub enum Geometry {
    Point([f64; 2]),
    BoundingBox(BoundingBox),
    ArcSegment(ArcSegment),
    LineSegment(LineSegment),
    Line(Line),
    Segment(Segment),
    Polysegment(Polysegment),
    Contour(Contour),
    Shape(Shape),
}
A container enum for owned geometric types. See the module-level documentation for more.

Variants



Point([f64; 2])

An owned point ([f64; 2]).



BoundingBox(BoundingBox)

An owned BoundingBox.



ArcSegment(ArcSegment)

An owned ArcSegment.



LineSegment(LineSegment)

An owned LineSegment.



Line(Line)

An owned Line.



Segment(Segment)

An owned Segment.



Polysegment(Polysegment)

An owned Polysegment.



Contour(Contour)

An owned Contour.



Shape(Shape)

An owned Shape.

Implementations

Source

impl Geometry

pub fn segment(&self, key: SegmentKey) -> Option<&Segment>

If self holds a Composite, returns a reference to the Segment specified by the given SegmentKey. Otherwise, returns None.

pub fn intersections<'b, T: Into<GeometryRef<'b>>>( &self, other: T, epsilon: f64, max_ulps: u32, ) -> Vec<Intersection>

Returns all intersections between self and other.

See GeometryRef::intersections for details and examples.

pub fn intersections_par<'b, T: Into<GeometryRef<'b>>>( &self, other: T, epsilon: f64, max_ulps: u32, ) -> Vec<Intersection>

Returns all intersections between self and other.

This is a parallelized version of Geometry::intersections, see its docstring for details. It uses parallel variants of the specialized intersection algorithms, if available.