pub struct Intersection<L, R> {
pub point: [f64; 2],
pub left: L,
pub right: R,
}Expand description
Intersection point and indices of the “left” and “right” composite types.
This struct is used as the Iterator::Item of the iterators created by the
intersection methods of the Composite trait. It contains the intersection
point itself as well as the segment keys of the involved composite types.
These keys (SegmentIdx or ShapeIdx) can be used to retrieve the segments
from the composite types where the intersection occurred (see
Composite::segment).
Fields
point: [f64; 2]The intersection point itself.
left: LKey to retrieve the intersected segment of the “left” side composite
(self for the intersection functions in Composite).
right: RKey to retrieve the intersected segment of the “right” side composite
(the second argument of the intersection functions in Composite).
Implementations
Sourceimpl<L, R> Intersection<L, R>
impl<L, R> Intersection<L, R>
Sourcepub fn switch(self) -> Intersection<R, L>
pub fn switch(self) -> Intersection<R, L>
Switches the left and ``rightfields ofSelf`.
Sourceimpl Intersection<ShapeIdx, ShapeIdx>
impl Intersection<ShapeIdx, ShapeIdx>
Sourcepub fn draw(
&self,
style: &IntersectionStyle,
left: Option<(&Shape, &Style)>,
right: Option<(&Shape, &Style)>,
context: &Context,
) -> Result<(), Error>
pub fn draw( &self, style: &IntersectionStyle, left: Option<(&Shape, &Style)>, right: Option<(&Shape, &Style)>, context: &Context, ) -> Result<(), Error>
Draws the intersection between two Shapes onto the given
cairo::Context using the given IntersectionStyle.
Optionally, the intersected segments of the involved composite types
SegmentChain, Contour or Shape can also
be drawn onto the canvas if the corresponding (&Composite, &Style) tuple
is provided. See the module documentation
for an example.
Sourceimpl Intersection<ShapeIdx, SegmentIdx>
impl Intersection<ShapeIdx, SegmentIdx>
Sourcepub fn draw(
&self,
style: &IntersectionStyle,
left: Option<(&Shape, &Style)>,
right: Option<(&SegmentChain, &Style)>,
context: &Context,
) -> Result<(), Error>
pub fn draw( &self, style: &IntersectionStyle, left: Option<(&Shape, &Style)>, right: Option<(&SegmentChain, &Style)>, context: &Context, ) -> Result<(), Error>
Draws the intersection between a Shape and a SegmentChain onto the
given cairo::Context using the given IntersectionStyle.
Optionally, the intersected segments of the involved composite types
SegmentChain, Contour or Shape can also
be drawn onto the canvas if the corresponding (&Composite, &Style) tuple
is provided. See the module documentation
for an example.
Sourceimpl Intersection<SegmentIdx, ShapeIdx>
impl Intersection<SegmentIdx, ShapeIdx>
Sourcepub fn draw(
&self,
style: &IntersectionStyle,
left: Option<(&SegmentChain, &Style)>,
right: Option<(&Shape, &Style)>,
context: &Context,
) -> Result<(), Error>
pub fn draw( &self, style: &IntersectionStyle, left: Option<(&SegmentChain, &Style)>, right: Option<(&Shape, &Style)>, context: &Context, ) -> Result<(), Error>
Draws the intersection between a SegmentChain and a Shape onto the
given cairo::Context using the given IntersectionStyle.
Optionally, the intersected segments of the involved composite types
SegmentChain, Contour or Shape can also
be drawn onto the canvas if the corresponding (&Composite, &Style) tuple
is provided. See the module documentation
for an example.
Sourceimpl Intersection<ShapeIdx, ()>
impl Intersection<ShapeIdx, ()>
Sourcepub fn draw(
&self,
style: &IntersectionStyle,
left: Option<(&Shape, &Style)>,
context: &Context,
) -> Result<(), Error>
pub fn draw( &self, style: &IntersectionStyle, left: Option<(&Shape, &Style)>, context: &Context, ) -> Result<(), Error>
Draws the intersection between a Shape and a
Primitive onto the given cairo::Context
using the given IntersectionStyle.
Optionally, the intersected segments of the involved composite types
SegmentChain, Contour or Shape can also
be drawn onto the canvas if the corresponding (&Composite, &Style) tuple
is provided. See the module documentation
for an example.
Sourceimpl Intersection<(), ShapeIdx>
impl Intersection<(), ShapeIdx>
Sourcepub fn draw(
&self,
style: &IntersectionStyle,
right: Option<(&Shape, &Style)>,
context: &Context,
) -> Result<(), Error>
pub fn draw( &self, style: &IntersectionStyle, right: Option<(&Shape, &Style)>, context: &Context, ) -> Result<(), Error>
Draws the intersection between a Primitive
and a Shape onto the given cairo::Context using the given
IntersectionStyle.
Optionally, the intersected segments of the involved composite types
SegmentChain, Contour or Shape can also
be drawn onto the canvas if the corresponding (&Composite, &Style) tuple
is provided. See the module documentation
for an example.
Sourceimpl Intersection<SegmentIdx, SegmentIdx>
impl Intersection<SegmentIdx, SegmentIdx>
Sourcepub fn draw(
&self,
style: &IntersectionStyle,
left: Option<(&SegmentChain, &Style)>,
right: Option<(&SegmentChain, &Style)>,
context: &Context,
) -> Result<(), Error>
pub fn draw( &self, style: &IntersectionStyle, left: Option<(&SegmentChain, &Style)>, right: Option<(&SegmentChain, &Style)>, context: &Context, ) -> Result<(), Error>
Draws the intersection between two SegmentChains onto the given
cairo::Context using the given IntersectionStyle.
Optionally, the intersected segments of the involved composite types
SegmentChain, Contour or Shape can also
be drawn onto the canvas if the corresponding (&Composite, &Style) tuple
is provided. See the module documentation
for an example.
Sourceimpl Intersection<SegmentIdx, ()>
impl Intersection<SegmentIdx, ()>
Sourcepub fn draw(
&self,
style: &IntersectionStyle,
left: Option<(&SegmentChain, &Style)>,
context: &Context,
) -> Result<(), Error>
pub fn draw( &self, style: &IntersectionStyle, left: Option<(&SegmentChain, &Style)>, context: &Context, ) -> Result<(), Error>
Draws the intersection between a SegmentChain and a
Primitive onto the given cairo::Context
using the given IntersectionStyle.
Optionally, the intersected segments of the involved composite types
SegmentChain, Contour or Shape can also
be drawn onto the canvas if the corresponding (&Composite, &Style) tuple
is provided. See the module documentation
for an example.
Sourceimpl Intersection<(), SegmentIdx>
impl Intersection<(), SegmentIdx>
Sourcepub fn draw(
&self,
style: &IntersectionStyle,
right: Option<(&SegmentChain, &Style)>,
context: &Context,
) -> Result<(), Error>
pub fn draw( &self, style: &IntersectionStyle, right: Option<(&SegmentChain, &Style)>, context: &Context, ) -> Result<(), Error>
Draws the intersection between a Primitive
and a SegmentChain onto the given cairo::Context using the given
IntersectionStyle.
Optionally, the intersected segments of the involved composite types
SegmentChain, Contour or Shape can also
be drawn onto the canvas if the corresponding (&Composite, &Style) tuple
is provided. See the module documentation
for an example.
Sourceimpl Intersection<(), ()>
impl Intersection<(), ()>
Sourcepub fn draw(
&self,
style: &IntersectionStyle,
context: &Context,
) -> Result<(), Error>
pub fn draw( &self, style: &IntersectionStyle, context: &Context, ) -> Result<(), Error>
Draws the intersection between two
Primitives onto the given
cairo::Context using the given IntersectionStyle.
Optionally, the intersected segments of the involved composite types
SegmentChain, Contour or Shape can also
be drawn onto the canvas if the corresponding (&Composite, &Style) tuple
is provided. See the module documentation
for an example.